/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ear.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.ee.deploy.api.deployable.CARDeployable;
import org.ow2.util.ee.deploy.api.deployable.EARDeployable;
import org.ow2.util.ee.deploy.api.deployable.EJB3Deployable;
import org.ow2.util.ee.deploy.api.deployable.LibDeployable;
import org.ow2.util.ee.deploy.api.deployable.WARDeployable;
import org.ow2.util.ee.deploy.api.deployable.metadata.DeployableMetadataException;
import org.ow2.util.ee.metadata.car.api.ICarDeployableMetadata;
import org.ow2.util.ee.metadata.car.impl.CarDeployableMetadataFactory;
import org.ow2.util.ee.metadata.ear.api.IEarDeployableMetadata;
import org.ow2.util.ee.metadata.ear.api.IEarDeployableMetadataFactory;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarDeployableMetadataFactory;
import org.ow2.util.ee.metadata.ejbjar.impl.internal.DefaultEjbJarDeployableFactory;
import org.ow2.util.ee.metadata.war.api.IWarDeployableMetadata;
import org.ow2.util.ee.metadata.war.impl.WarDeployableMetadataFactory;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.classlocator.ArchiveClassesLocator;
import org.ow2.util.scan.api.classlocator.ClassLoaderClassesLocator;
import org.ow2.util.scan.impl.ASMScannerImpl;
import org.ow2.util.url.URLUtils;
import org.ow2.util.url.URLUtilsException;

/**
 * Implementation of ear deployable metadata.
 * @author Gael Lalire
 */
public class EarDeployableMetadataFactory implements IEarDeployableMetadataFactory {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(EarDeployableMetadataFactory.class);

    /**
     * The factory which manage war.
     */
    private WarDeployableMetadataFactory warDeployableMetadataFactory;

    /**
     * The factory which manage client archive.
     */
    private CarDeployableMetadataFactory carDeployableMetadataFactory;

    /**
     * The factory which manage ejbjar.
     */
    private EjbJarDeployableMetadataFactory<EJB3Deployable, ?, ?, ?, ?> ejbJarDeployableMetadataFactory;

    /**
     * Constructor. Use {@link ASMScannerImpl} to analyze class bytecode.
     */
    public EarDeployableMetadataFactory() {
        this(new ASMScannerImpl());
    }

    /**
     * Constructor.
     * @param scanner the scanner which analyze class bytecode.
     */
    @SuppressWarnings("unchecked")
    public EarDeployableMetadataFactory(final IScanner scanner) {
        warDeployableMetadataFactory = new WarDeployableMetadataFactory(scanner);
        carDeployableMetadataFactory = new CarDeployableMetadataFactory(scanner);
        ejbJarDeployableMetadataFactory = new DefaultEjbJarDeployableFactory<EJB3Deployable>(scanner);
    }

    /**
     * @param deployable the ear deployable (getOriginalDeployable must not be
     *        null ie it's already decompressed archive)
     * @return a new ear deployable metadata
     * @throws DeployableMetadataException if ear is not uncompressed
     */
    public IEarDeployableMetadata createDeployableMetadata(final EARDeployable deployable) throws DeployableMetadataException {
        return createDeployableMetadata(deployable, null);
    }

    /**
     * @param deployable the ear deployable (getOriginalDeployable must not be
     *        null ie it's already decompressed archive)
     * @param classLoader a classloader to use if a class is not in archive
     * @return a new ear deployable metadata
     * @throws DeployableMetadataException if ear is not uncompressed
     */
    public IEarDeployableMetadata createDeployableMetadata(final EARDeployable deployable, final ClassLoader classLoader)
            throws DeployableMetadataException {

        long tAnalyzeStart = System.currentTimeMillis();

        try {
            if (!URLUtils.urlToFile2(deployable.getArchive().getURL()).isDirectory()) {
                throw new DeployableMetadataException("ear must be uncompressed (local directory expected)");
            }
        } catch (URLUtilsException e) {
            throw new DeployableMetadataException("ear must be uncompressed (local directory expected)");
        } catch (ArchiveException e) {
            throw new DeployableMetadataException("ear exception", e);
        }
        List<LibDeployable> libDeployableList = deployable.getLibDeployables();
        List<IClassesLocator> libraryList = new ArrayList<IClassesLocator>();
        for (LibDeployable libDeployable : libDeployableList) {
            libraryList.add(new ArchiveClassesLocator(libDeployable.getArchive()));
        }
        if (classLoader != null) {
            libraryList.add(new ClassLoaderClassesLocator(classLoader));
        } else {
            libraryList.add(new ClassLoaderClassesLocator(ClassLoader.getSystemClassLoader()));
        }
        EarDeployableMetadata earDeployableMetadata = new EarDeployableMetadata(deployable);

        // war
        for (WARDeployable warDeployable : deployable.getWARDeployables()) {
            IWarDeployableMetadata warDeployableMetadata = warDeployableMetadataFactory.createDeployableMetadata(warDeployable,
                    libraryList, null);
            earDeployableMetadata.addWarDeployableMetadata(warDeployableMetadata);
        }

        // car
        for (CARDeployable carDeployable : deployable.getCARDeployables()) {
            ICarDeployableMetadata carDeployableMetadata = carDeployableMetadataFactory.createDeployableMetadata(carDeployable,
                    libraryList, null);
            earDeployableMetadata.addCarDeployableMetadata(carDeployableMetadata);
        }

        // ejb jar
        for (EJB3Deployable ejbJarDeployable : deployable.getEJB3Deployables()) {
            IEjbJarDeployableMetadata<EJB3Deployable, ?, ?, ?, ?> ejbJarDeployableMetadata = ejbJarDeployableMetadataFactory
                    .createDeployableMetadata(ejbJarDeployable, libraryList);
            earDeployableMetadata.addEjbJarDeployableMetadata(ejbJarDeployableMetadata);
        }

        if (LOGGER.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            LOGGER.debug("Analyze of file {0} took {1} ms.", deployable.getArchive().getName(),
                    (tAnalyzeStartEnd - tAnalyzeStart));
        }

        return earDeployableMetadata;
    }

    /**
     * @return the supported class of this factory
     */
    public Class<IEarDeployableMetadata> getDeployableMetadataClass() {
        return IEarDeployableMetadata.class;
    }

}
